unit Main_U ;

(*
   Annotated Player.
   =================

  Syfte:
    Vid t.ex. sprkundervisning vill man kunna kontrollera 
    uppspelningen av delar av en ljudfil. Det skall g ltt 
    och snabbt att hoppa mellan olika avsnitt, t.ex. fr att 
    repetera en passage.

  Beskrivning:
    En textfil anger vilken ljudfil som skall spelas upp. 
    I textfilen finns ocks angivet olika tidpunkter i 
    ljudfilen dr olika avsnitt startar. Dessa tider kan 
    anges med sekundupplsning, t.ex. 12:35 fr en position
    12 minuter och 35 sekunder in i ljudfilen. Genom att 
    dubbelklicka p en tidangivelse startar uppspelningen 
    av ljudfilen p detta stlle.

  Programmet kan arbeta med Windows wav-filer. Om Windows 
  Media Player 2 (gratis frn Microsoft) eller senare 
  installerats kan ven uppspelning av mp3-filer kontrolleras.

  Fr ev. uppdateringar och rttelser - se:
    http://home5.swipnet.se/~w-54755/gf1.htm

  Funktioner:
    - Lsa in, redigera och spara textfil.
    - Den frsta raden i textfilen = "LJUDFIL=ljudfilsnamn"
       P detta stt kopplas textfilen och ljudfilen ihop.
    - "Avspelning"-knapp ger manuell uppspelning av ljudfil.
    - "Backa"-knapp ger terhopp 10 sek. i ljudfilen.
    - "Markera Tid"-knapp lgger in inspelningens tid i texten
       och avbryter uppspelningen. Hr kan man nu skriva in sina 
       egna kommentarer fr att ltt hitta samma avsnitt igen. 
       Det gr ocks att skriva in dessa tidangivelser manuellt.
    - D en tidangivelse dubbelklickas i texten startas 
      avspelning p denna position i ljudfilen.
      Tidangivelse= hh:mm:ss eller mm:ss 
      hh=timme, mm=minut, ss=sekund Ex. 01:00:10, 01:30, 00:15
    - Parameter "/F=filnamn" anvnds fr att starta med en fil
      inlst direkt vid start frn t.ex. en "shortcut" i Windows.
    - Parameter "/P=hh:mm:ss" anvnds om man vill starta 
      avspelningen direkt frn angiven tid.

   Frfattare: Gran Forsstrm, Humlegatan 16C, 722 26 Vsters.

   1999-04-28/GF: Introducerad.
   1999-04-29/GF: Adderat parameter "/P=".


*)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Menus, MPlayer;

CONST
  C_LONG_VERSION   = 'Annotated Player v.1.0 1999-04-29/GF' ;
  C_SHORT_VERSION  = 'Ann.Player' ;

type
  TfrmMain = class(TForm)
    Panel1: TPanel;
    Memo1: TMemo;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Help1: TMenuItem;
    New1: TMenuItem;
    Open1: TMenuItem;
    Save1: TMenuItem;
    SaveAs1: TMenuItem;
    N1: TMenuItem;
    Exit1: TMenuItem;
    Description1: TMenuItem;
    N2: TMenuItem;
    About1: TMenuItem;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    MediaPlayer1: TMediaPlayer;
    cmdSelectPlayFile: TButton;
    OpenDialog2: TOpenDialog;
    cmdMark: TButton;
    edtPlayTime: TEdit;
    Timer1: TTimer;
    cmdPlay: TButton;
    edtTotalTime: TEdit;
    cmdStop: TButton;
    cmdRepeatLastSecs: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure New1Click(Sender: TObject);
    procedure Open1Click(Sender: TObject);
    procedure Save1Click(Sender: TObject);
    procedure SaveAs1Click(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure cmdSelectPlayFileClick(Sender: TObject);
    procedure Memo1Change(Sender: TObject);
    procedure cmdMarkClick(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure cmdPlayClick(Sender: TObject);
    procedure Memo1DblClick(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure Description1Click(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure cmdStopClick(Sender: TObject);
    procedure cmdRepeatLastSecsClick(Sender: TObject);
  private
    { Private declarations }
    FIsAutoLoad    : boolean ;    (* TRUE innebr att filnamn getts vid starten.   *)
    FAutoFileName  : string ;     (* Namn p fil i "/F=filnamn".                   *)
    FIsAutoPlay    : boolean ;    (* TRUE innebr att avspelning startas direkt.   *)
    FAutoPlayTime  : longiNT ;    (* Start tid fr autoplay i sekunder.            *)
    FSaveIsNeeded  : boolean ;    (* TRUE d fil ej sparad.                        *)
    FSaveFileName  : string ;     (* Blankt namn innebr att namn ej getts nnu.   *)
    PROCEDURE DecodeParameters ;
    PROCEDURE SoundFilePlay (CONST i_PositionMS : longInt) ;
    PROCEDURE SoundFileLoad (CONST i_FileName : string) ;
    PROCEDURE TextFileRead (CONST i_FileName : string) ;
    PROCEDURE TextFileSave (CONST i_FileName : string) ;
 public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

uses
  About_U,
  Info_U;

{$R *.DFM}

CONST
  C_SOUND_TAG      = 'LJUDFIL=' ;

TYPE
 ETimeFormat       = CLASS (Exception) ;

FUNCTION SecToStr (CONST i_Secs : longInt) : string ;
(* Konvertera sekund-tal till strng HH:MM:SS. *)
VAR
  h : integer ;
  m : integer ;
  s : integer ;
BEGIN
  h := i_Secs DIV 3600 ;          (* Timme *)
  m := (i_Secs DIV 60) MOD 60 ;   (* Minut *)
  s := i_Secs MOD 60 ;            (* Sekund *)
  Result := Format ('%2.2d:%2.2d:%2.2d', (.h, m, s.)) ;
END ; (* SecToStr *)

FUNCTION HHMMSSStrToInt (CONST i_HHMMSSStr : string) : integer ;
(* Konvertera tidangivelse i strngform 'hh:mm:ss' till sekunder.
   NB "hh:m" optional. Inledande blanka och () OK.
   ex. 1:12:13, 10:33, 7:32, 0:01
*)
VAR
  s           : string ;
  p           : integer ;
  i           : integer ;
  h           : longInt ;
  m           : 0..59 ;
  sec         : 0..59 ;
  colCount    : integer ;
  colPosArr   : ARRAY (.1..2.) OF integer ;
  hPos        : integer ;
  mPos        : integer ;
  sPos        : integer ;
BEGIN
  Result := 0 ;

  (* Tag bort alla ' ', '(', ')' *)
  s := i_HHMMSSStr ;
  REPEAT
    p := Pos ('(', s) ;
    IF p = 0 THEN
      p := Pos (')', s) ;
    IF p = 0 THEN
      p := Pos (' ', s) ;
    IF p > 0 THEN
      Delete (s, p, 1) ;
  UNTIL p = 0 ;

  colCount := 0 ;
  colPosArr (.1.) := 0 ;
  colPosArr (.2.) := 0 ;

  FOR i := 1 TO Length (s) DO
  BEGIN
    IF s (.i.) = ':' THEN
    BEGIN
      Inc (colCount) ;
      IF colCount > 2 THEN
        Break ;
      colPosArr (.colCount.) := i ;
    END ;
  END ; (* for *)

  IF (colCount < 1) OR (colCount > 2) THEN
  BEGIN
    Raise ETimeFormat.Create ('Skall vara "hh:mm:ss" eller "mm:ss", inte "' + i_HHMMSSStr + '"') ;
  END ;

  IF colCount = 1 THEN
  BEGIN
    (* 'mm:ss' eller 'm:ss' *)
    hPos := 0 ;
    mPos := 1 ;
    sPos := colPosArr (.1.) + 1 ;
  END ;
  IF colCount = 2 THEN
  BEGIN
    (* 'hh:mm:ss' eller 'h:mm:ss' *)
    hPos := 1 ;
    mPos := colPosArr (.1.) + 1 ;
    sPos := colPosArr (.2.) + 1 ;
  END ;

  TRY
    h := 0 ;
    IF hPos > 0 THEN
      h := StrToInt (Copy (s, hPos, mPos - hPos -1)) ;
    m := StrToInt (Copy (s, mPos, sPos - mPos - 1)) ;
    sec := StrToInt (Copy (s, sPos, 2)) ;
  EXCEPT
    Raise ETimeFormat.Create ('Frvntat hh:mm:ss/mm:ss - ej: "' + i_HHMMSSStr + '"') ;
  END ;
  Result := h * 60 * 60 + m * 60 + sec ;
END ; (* HHMMSSStrToInt *)

(* -------------------------------------------------------------------------- *)

PROCEDURE TfrmMain.DecodeParameters ;
(* Undersk om ngon parameter = "/F=filnamn" *)
VAR
  i : integer ;
  s : string ;
BEGIN
  FIsAutoLoad    := FALSE ;
  FAutoFileName  := '' ;
  FIsAutoPlay    := FALSE ;
  FAutoPlayTime  := 0 ;
  FOR i := 1 TO ParamCount DO
  BEGIN
    s := ParamStr (i) ;
    s := UpperCase (s) ;
    IF Pos ('/F=', s) = 1 THEN
    BEGIN
      FIsAutoLoad    := TRUE ;
      FAutoFileName  := Copy (ParamStr (i), 4, 255) ;
    END ;
    IF Pos ('/P=', s) = 1 THEN
    BEGIN
      FIsAutoPlay   := TRUE ;
      FAutoPlayTime := HHMMSSStrToInt (Copy (ParamStr (i), 4, 255)) ;
    END ;
  END ; (* for i *)

  (* Bara autoplay om autostart. *)
  IF NOT FIsAutoLoad THEN
    FIsAutoPlay := FALSE ;
END ; (* DecodeParameters *)

PROCEDURE TfrmMain.SoundFilePlay (CONST i_PositionMS : longInt) ;
(* Starta uppspelning p den angivna tiden. *)
BEGIN
  IF (i_PositionMS < 0) OR (i_PositionMS > Mediaplayer1.Length) THEN
  BEGIN
    ShowMessage ('Angiven tid utanfr inspelninges lngd!') ;
    Exit ;
  END ;

  TRY
    edtPlayTime.Text  := 'Vnta...' ;
    edtPlayTime.Color := clYellow ;
    Application.ProcessMessages ; (* "Play" kan ta tid... *)
    TRY
      Mediaplayer1.Position := i_PositionMS ;
      Mediaplayer1.Play ;
    EXCEPT
      ShowMessage ('Problem att spela upp ljudet - kontrollera filtyp!') ;
      Raise ;
    END ;
  FINALLY
    edtPlayTime.Color := clWindow ;
    edtPlayTime.Text  := '' ;
  END ;
END ; (* SoundFilePlay *)

PROCEDURE TfrmMain.SoundFileLoad (CONST i_FileName : string) ;
(* Ladda ljudfilen i MediaPlayer.
   Uppdatera "LJUDFIL="-raden med ljudfilsnamnet.
*)

VAR
  s : string ;
  p : integer ;
BEGIN
  IF NOT FileExists (i_FileName) THEN
  BEGIN
    ShowMessage ('Ljudfil "' + i_FileName + '" hittas ej!') ;
    Exit ;
  END ;

  TRY
    TRY
      MediaPlayer1.FileName := i_FileName ;
      MediaPlayer1.TimeFormat := tfMilliseconds ;
      edtPlayTime.Text := 'Vnta...' ;
      edtPlayTime.Color := clYellow ;
      Application.ProcessMessages ; (* "Open" kan ta tid... *)
      MediaPlayer1.Open ;
      edtPlayTime.Color := clWindow ;
      edtTotalTime.Text := SecToStr (MediaPlayer1.Length DIV 1000) ;
      Timer1.Enabled := TRUE ;

      (* Uppdatera "LJUDFIL="-raden - skall vara frst i filen! *)
      IF Memo1.Lines.Count > 0 THEN
      BEGIN
        s := Memo1.Lines (.0.) ;
        p := Pos (C_SOUND_TAG, s) ;
        IF p <> 0 THEN
          Memo1.Lines.Delete (0) ;
      END ;
      s :=C_SOUND_TAG + i_FileName ;
      Memo1.Lines.Insert (0, s) ;
    EXCEPT
      ShowMessage ('Kan ej ladda ljudfil "' + i_FileName + '" - fil korrupt?') ;
    END ;
  FINALLY
    edtPlayTime.Color := clWindow ;
    edtPlayTime.Text  := '' ;
  END ;
END ; (* SoundFileLoad *)

PROCEDURE TfrmMain.TextFileRead (CONST i_FileName : string) ;
VAR
  s : string ;
  p : integer ;
BEGIN
  IF NOT FileExists (i_FileName) THEN
  BEGIN
    ShowMessage ('Textfil "' + i_FileName + '" hittas ej!') ;
    Exit ;
  END ;

  TRY
    Memo1.Lines.LoadFromFile (i_FileName) ;
    FSaveFileName        := i_FileName ;
    SaveDialog1.FileName := FSaveFileName ;
    Caption              := FSaveFileName + ' - ' + C_SHORT_VERSION ;
  EXCEPT
    ShowMessage ('Kan ej lsa textfil "' + i_FileName + '" - fil korrupt?') ;
    Exit ;
  END ;

  (* Ladda ljudfilen - om ngon har angets. *)
  IF Memo1.Lines.Count > 0 THEN
  BEGIN
    s := Memo1.Lines (.0.) ;
    p := Pos (C_SOUND_TAG, s) ;
    IF p <> 0 THEN
      SoundFileLoad (Copy (s, p + Length (C_SOUND_TAG), 255)) ;
  END ;

  FSaveIsNeeded := FALSE ;

END ; (* TextFileRead *)

PROCEDURE TfrmMain.TextFileSave (CONST i_FileName : string) ;
BEGIN
  TRY
    FSaveFileName := i_FileName ;
    Memo1.Lines.SaveToFile (i_FileName) ;
    FSaveIsNeeded := FALSE ;
    Caption       := FSaveFileName + ' - ' + C_SHORT_VERSION ;
  EXCEPT
    ShowMessage ('Kan ej spara fil "' + i_FileName + '" - utrymme slut? Skrivskydd?') ;
  END ;
END ; (* TextFileSave *)

(* ========================================================================== *)

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  Memo1.Clear ;
  edtPlayTime.Text       := '' ;
  edtTotalTime.Text      := '' ;
  Caption                := C_LONG_VERSION ;
  OpenDialog1.InitialDir := ExtractFilePath (Application.ExeName) ;
  SaveDialog1.InitialDir := OpenDialog1.InitialDir ;
  OpenDialog2.InitialDir := ExtractFilePath (Application.ExeName) ;
  DecodeParameters ;
end; (* FormCreate *)

procedure TfrmMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  IF FSaveIsNeeded THEN
  BEGIN
    IF MessageDlg ('Ej sparad - vill du avsluta i alla fall?',
                   mtConfirmation,
                   [mbYes, mbNo], 0) = mrYes THEN
      Action := caFree
    ELSE
      Action := caNone ;
  END ;
end; (* FormClose *)

procedure TfrmMain.New1Click(Sender: TObject);
begin
  IF FSaveIsNeeded THEN
  BEGIN
    IF MessageDlg ('Ej sparad - vill du brja med en ny fil i alla fall?',
                   mtConfirmation,
                   [mbYes, mbNo], 0) = mrNo THEN
      Exit ;
  END ;
  FSaveIsNeeded := FALSE ;
  FSaveFileName := '' ;
  Caption       := '[Namn Saknas] - ' + C_SHORT_VERSION ;
  SaveDialog1.FileName := 'noname.txt' ;
  Memo1.Clear ;
end; (* New1Click *)

procedure TfrmMain.Open1Click(Sender: TObject);
begin
  IF FSaveIsNeeded THEN
  BEGIN
    IF MessageDlg ('Ej sparad - vill du lsa in en ny fil i alla fall?',
                   mtConfirmation,
                   [mbYes, mbNo], 0) = mrNo THEN
      Exit ;
  END ;

  IF OpenDialog1.Execute THEN
  BEGIN
     TextFileRead (OpenDialog1.FileName) ;
  END ;
end; (* Open1Click *)

procedure TfrmMain.Save1Click(Sender: TObject);
begin
  IF FSaveFileName = '' THEN
    SaveAs1Click (Self)
  ELSE
    TextFileSave (FSaveFileName) ;
end; (* Save1Click *)

procedure TfrmMain.SaveAs1Click(Sender: TObject);
begin
  IF SaveDialog1.Execute THEN
  BEGIN
    TextFileSave (SaveDialog1.FileName) ;
  END ;
end; (* SaveAs1Click *)

procedure TfrmMain.Exit1Click(Sender: TObject);
begin
  Close ;
end; (* Exit1Click *)

procedure TfrmMain.cmdSelectPlayFileClick(Sender: TObject);
begin
  IF OpenDialog2.Execute THEN
  BEGIN
    SoundFileLoad (OpenDialog2.FileName) ;
  END ;
end; (* cmdSelectPlayFileClick *)

procedure TfrmMain.Memo1Change(Sender: TObject);
begin
  FSaveIsNeeded := TRUE ;
end; (* Memo1Change *)

procedure TfrmMain.cmdMarkClick(Sender: TObject);
(* Stanna avspelningen, ls av tiden och kopiera in i texten. *)
VAR
  s : string ;
begin
  MediaPlayer1.Pause ;
  s := edtPlayTime.Text ;
  edtPlayTime.Text := s + ' - ' ;
  edtPlayTime.SelectAll ;
  edtPlayTime.CopyToClipboard ;
  Memo1.PasteFromClipboard ;
  edtPlayTime.Text := s ;
end; (* cmdMarkClick *)

procedure TfrmMain.Timer1Timer(Sender: TObject);
(* Presentera spelad tid cykliskt. *)
VAR
  p : longInt ;
begin
  p := MediaPlayer1.Position ; (* Millisekunder. *)
  p := p DIV 1000 ;            (* Sekunder *)
  edtPlayTime.Text := SecToStr (p) ;
end; (* Timer1Timer *)

procedure TfrmMain.cmdPlayClick(Sender: TObject);
begin
  SoundFilePlay (MediaPlayer1.Position) ;
end; (* cmdPlayClick *)

procedure TfrmMain.Memo1DblClick(Sender: TObject);
(* Starta uppspelning p den klickade tiden. *)
VAR
  s : string ;
  secs : longInt ;
begin
  s := Memo1.SelText ;
  secs := HHMMSSStrToInt (s) ;
  SoundFilePlay (1000 * secs) ;
end; (* Memo1DblClick *)

procedure TfrmMain.About1Click(Sender: TObject);
begin
  frmAboutBox.lblProductName.Caption := C_LONG_VERSION ;
  frmAboutBox.lblAuthor.Caption      := 'G.Forsstrm, Vsters' ;
  frmAboutBox.lblCopyright.Caption   := '(c) 1999' ;
  frmAboutBox.memComments.Clear ;
  frmAboutBox.memComments.Lines.Add ('Uppdateringar - se:') ;
  frmAboutBox.memComments.Lines.Add ('') ;
  frmAboutBox.memComments.Lines.Add ('http://home5.swipnet.se/~w-54755/gf1.htm') ;
  frmAboutBox.ShowModal ;
end; (* About1Click *)

procedure TfrmMain.Description1Click(Sender: TObject);
begin
  frmInfo.Caption := C_LONG_VERSION ;
  frmInfo.Show ;
end; (* Description1Click *)

procedure TfrmMain.FormActivate(Sender: TObject);
(* Starta med fil frn start-parameter - om sdan angetts. *)
begin
  IF FIsAutoLoad THEN
  BEGIN
    TextFileRead (FAutoFileName) ;
    IF FIsAutoPlay THEN
      SoundFilePlay (1000* FAutoPlayTime) ;
  END ;
  FIsAutoLoad := FALSE ; (* Skall bara gras en gng. *)
  FIsAutoPlay := FALSE ;
end;

procedure TfrmMain.cmdStopClick(Sender: TObject);
begin
  MediaPlayer1.Stop ;
end;

procedure TfrmMain.cmdRepeatLastSecsClick(Sender: TObject);
VAR
  p : longInt ;
begin
  p := MediaPlayer1.Position ;
  (* Backa 10 sek. bakt - om mjligt. *)
  p := p - 10000 ;
  IF p < 0 THEN
    p := 0 ;
  SoundFilePlay (p) ;
end;

end.
